"""Tests for distutils.pypirc.pypirc."""
import sys
import os
import unittest

from collective.dist.core import PyPIRCCommand
from setuptools.dist import Distribution

from distutils.tests import support

PYPIRC = """\
[distutils]

index-servers =
    server1
    server2

[server1]
username:me
password:secret

[server2]
username:meagain
password: secret
realm:acme
repository:http://another.pypi/
"""

PYPIRC_OLD = """\
[server-login]
username:tarek
password:secret
"""

SETUP_CFG = """\
[distutils]
index-servers =
    server3

[server3]
username:gawel
password: pipi
"""

class PyPIRCCommandTestCase(support.TempdirManager, unittest.TestCase):

    def setUp(self):
        """Patches the environment."""
        if os.environ.has_key('HOME'):
            self._old_home = os.environ['HOME']
        else:
            self._old_home = None
        curdir = os.path.dirname(__file__)
        os.environ['HOME'] = curdir
        self.rc = os.path.join(curdir, '.pypirc')
        self.setup_cfg = os.path.join(curdir, 'setup.cfg')
        self.dist = Distribution()

        class command(PyPIRCCommand):
            def __init__(self, dist):
                PyPIRCCommand.__init__(self, dist)
            def initialize_options(self):
                pass
            finalize_options = initialize_options

        self._cmd = command

    def tearDown(self):
        """Removes the patch."""
        if self._old_home is None:
            del os.environ['HOME']
        else:
            os.environ['HOME'] = self._old_home
        if os.path.exists(self.rc):
            os.remove(self.rc)
        if os.path.exists(self.setup_cfg):
            os.remove(self.setup_cfg)

    def test_server_registration(self):
        # This test makes sure PyPIRCCommand knows how to:
        # 1. handle several sections in .pypirc
        # 2. handle the old format

        # new format
        f = open(self.rc, 'w')
        try:
            f.write(PYPIRC)
        finally:
            f.close()

        cmd = self._cmd(self.dist)
        config = cmd._read_pypirc()

        config = config.items()
        config.sort()
        waited = [('password', 'secret'), ('realm', 'pypi'),
                  ('repository', 'http://pypi.python.org/pypi'),
                  ('server', 'server1'), ('username', 'me')]
        self.assertEquals(config, waited)

        # old format
        f = open(self.rc, 'w')
        f.write(PYPIRC_OLD)
        f.close()

        config = cmd._read_pypirc()
        config = config.items()
        config.sort()
        waited = [('password', 'secret'), ('realm', 'pypi'),
                  ('repository', 'http://pypi.python.org/pypi'),
                  ('server', 'server-login'), ('username', 'tarek')]
        self.assertEquals(config, waited)

    def test_setup_cfg(self):

        # assume that we are in a test directory
        pwd = os.getcwd()
        os.chdir(os.environ['HOME'])

        # write setup.cfg
        f = open(self.setup_cfg, 'w')
        f.write(SETUP_CFG)
        f.close()

        cmd = self._cmd(self.dist)
        config = cmd._read_pypirc()
        config = config.items()
        config.sort()

        waited = [('password', 'pipi'), ('realm', 'pypi'),
                  ('repository', 'http://pypi.python.org/pypi'),
                  ('server', 'server3'), ('username', 'gawel')]
        self.assertEquals(config, waited)

        # assume that the setup.cfg override the .pypirc

        # write .pypirc
        f = open(self.rc , 'w')
        f.write(PYPIRC)
        f.close()

        cmd = self._cmd(self.dist)
        config = cmd._read_pypirc()
        config = config.items()
        config.sort()

        waited = [('password', 'pipi'), ('realm', 'pypi'),
                  ('repository', 'http://pypi.python.org/pypi'),
                  ('server', 'server3'), ('username', 'gawel')]
        self.assertEquals(config, waited)

        os.chdir(pwd)


    def test_get_home(self):
        # environment variable can differ depending on the platform
        # so the rule is:
        # 1. get HOME
        # 2. get HOMEDIR+HOMEPATH
        # 3. get working dir
        cmd = self._cmd(self.dist)
        curdir = os.path.dirname(__file__)

        # 1.
        self.assertEquals(cmd._get_home(), curdir)

        # 2.
        del os.environ['HOME']

        os.environ['HOMEPATH'] = os.path.dirname(__file__)
        os.environ['HOMEDRIVE'] = 'the_drive'
        self.assertEquals(cmd._get_home(), 'the_drive' + curdir)

        # 3.
        del os.environ['HOMEDRIVE']
        del os.environ['HOMEPATH']
        self.assertEquals(cmd._get_home(), os.curdir)

        # setting back home
        os.environ['HOME'] = curdir

    def test_repository_in_args(self):
        # make sure the -r option is always caught

        sys.argv.extend(['-r', 'toto'])
        try:
            cmd = PyPIRCCommand(self.dist)
            cmd.finalize_options()
            self.assertEquals(cmd.repository, 'toto')
        finally:
            sys.argv = sys.argv[:-2]

def test_suite():
    return unittest.makeSuite(PyPIRCCommandTestCase)

if __name__ == "__main__":
    unittest.main(defaultTest="test_suite")



"""Tests for distutils.command.register."""
import sys
import os
import unittest
from StringIO import StringIO

from collective.dist.mregister import mregister
from setuptools.dist import Distribution

from distutils.tests import support
from collective.dist.tests.test_pypirc import PYPIRC, PyPIRCCommandTestCase

import urllib2

data_sent = {}

class Opener(object):
    
    def open(self, req, data=None):
        if isinstance(req, str):
            assert req.startswith('http')
            return StringIO('ok')
        data_sent[req.get_host()] = (req.headers, req.data)

def build_opener(*auth):
    return Opener()

urllib2.build_opener = build_opener

class registerTestCase(PyPIRCCommandTestCase):

    def setUp(self):
        PyPIRCCommandTestCase.setUp(self)
        f = open(self.rc, 'w')
        f.write(PYPIRC)
        f.close()
        self.dist = Distribution()
        self.cmd = mregister(self.dist)
        
    def test_mregister(self):
        
        self.cmd.send_metadata()
        length = data_sent['pypi.python.org'][0]['Content-length']
        self.assertEquals(length, '1392')

    def test_get_non_default(self):
        # tests if we send data to the non default
        self.cmd.repository = 'server2'
        self.cmd.send_metadata()
        self.assert_('another.pypi' in data_sent)

        # tests if we send data to the non default
        import urllib2
        old = urllib2.HTTPPasswordMgr
        class _HTTPPasswordMgr:
            def add_password(self, realm, *args):
                urllib2._realm = realm
        urllib2.HTTPPasswordMgr = _HTTPPasswordMgr


        self.cmd.repository = 'server2'
        self.cmd.send_metadata()
        self.assert_('another.pypi' in data_sent)
        self.assertEquals(urllib2._realm, 'acme')
        urllib2.HTTPPasswordMgr = old
        delattr(urllib2, '_realm')

    def test_not_existing(self):
        # making sure we get an explicit message when the server 
        # does not exists in the .pypirc file
        self.cmd.repository = 'server128'
        self.assertRaises(ValueError, self.cmd.send_metadata)

    def test_creating_pypirc(self):
        # making sure we create a pypirc
        os.remove(self.cmd._get_rc_file())

        self.cmd.repository = 'pypi'

        # patching things to avoid interaction
        def _raw_input(msg=''):
            if msg == 'Save your login (y/N)?':
                return 'y'
            return '1'
        import getpass
        getpass.getpass = _raw_input
        func_globs = self.cmd.send_metadata.im_func.func_globals
        func_globs['raw_input'] = _raw_input

        # calling the register
        self.cmd.send_metadata()

        # now checking the created file
        content = open(self.cmd._get_rc_file()).read()
        content = content.replace(' ', '')
        wanted = """\
        [pypirc]
        servers = 
            pypi
                
        [pypi]
        username:1
        password:1""".strip()

        self.assertEquals(content.strip(), wanted.replace(' ', ''))

    def test_classifiers(self):
        # when calling the classifier with the contracted
        # -r option, it is not translated
        self.cmd.repository = 'server1' 
        self.cmd.classifiers()
        

def test_suite():
    return unittest.makeSuite(registerTestCase)

if __name__ == "__main__":
    unittest.main(defaultTest="test_suite")




"""distutils.pypirc

Provides the PyPIRcCommand class, the base class for the command classes
that uses .pypirc in the distutils.command package.
"""
import os
import sys
from ConfigParser import ConfigParser

from distutils.core import Command

DEFAULT_PYPIRC = """\
[pypirc]
servers =
    pypi

[pypi]
username:%s
password:%s
"""

class PyPIRCCommand(Command):
    """Base command that knows how to handle the .pypirc file
    """
    DEFAULT_REPOSITORY = 'http://pypi.python.org/pypi'
    DEFAULT_REALM = 'pypi'
    repository = None
    realm = None

    def _get_rc_file(self):
        """Returns rc file path."""
        return os.path.join(self._get_home(), '.pypirc')

    def _get_home(self):
        """Returns the home directory.

        home can differ depending on the platform"""
        home = os.getenv('HOME')    # linux style
        if home is None:
            home_drive = os.getenv('HOMEDRIVE') # win32 style
            home_path = os.getenv('HOMEPATH')
            if home_drive is not None and home_path is not None:
                return home_drive + home_path
        else:
            return home
        return os.curdir

    def _store_pypirc(self, username, password):
        """Creates a default .pypirc file."""
        rc = self._get_rc_file()
        f = open(rc, 'w')
        try:
            f.write(DEFAULT_PYPIRC % (username, password))
        finally:
            f.close()
        try:
            os.chmod(rc, 0600)
        except OSError:
            # should do something better here
            pass

    def _read_pypirc(self):
        """Reads the .pypirc file and setup.cfg if any."""
        rc = self._get_rc_file()
        files = [f for f in [rc, 'setup.cfg'] if os.path.exists(f)]
        if files:
            repository = self.repository or self.DEFAULT_REPOSITORY
            realm = self.realm or self.DEFAULT_REALM

            print 'Using PyPI login from %s' % ', '.join(files)
            config = ConfigParser()
            config.read(files)

            sections = config.sections()
            if 'distutils' in sections:
                # let's get the list of servers
                index_servers = config.get('distutils', 'index-servers')
                _servers = [server.strip() for server in
                            index_servers.split('\n')
                            if server.strip() != '']
                if _servers == []:
                    # nothing set, let's try to get the defaut pypi
                    if 'pypi' in sections:
                        _servers = ['pypi']
                    else:
                        # the file is not properly defined, returning
                        # an empty dict
                        return {}
                for server in _servers:
                    current = {'server': server}
                    current['username'] = config.get(server, 'username')
                    current['password'] = config.get(server, 'password')

                    # optional params
                    for key, default in (('repository',
                                          self.DEFAULT_REPOSITORY),
                                         ('realm', self.DEFAULT_REALM)):
                        if config.has_option(server, key):
                            current[key] = config.get(server, key)
                        else:
                            current[key] = default
                    if (current['server'] == repository or
                        current['repository'] == repository):
                        return current
            elif 'server-login' in sections:
                # old format
                server = 'server-login'
                if config.has_option(server, 'repository'):
                    repository = config.get(server, 'repository')
                else:
                    repository = self.DEFAULT_REPOSITORY
                return {'username': config.get(server, 'username'),
                        'password': config.get(server, 'password'),
                        'repository': repository,
                        'server': server,
                        'realm': self.DEFAULT_REALM}

        return {}

    def initialize_options(self):
        self.repository = None
        self.realm = None

    def finalize_options(self):
        # we need to make sure the repository is not provided
        # after another command in the arguments
        # if so, we need to get it
        if '-r' in sys.argv or '--repository' in sys.argv:
            if '-r' in sys.argv:
                 pos = sys.argv.index('-r') + 1
            else:
                pos = sys.argv.index('--repository') + 1
            if len(sys.argv) > pos:
                self.repository = sys.argv[pos]

        if self.repository is None:
            self.repository = self.DEFAULT_REPOSITORY
        if self.realm is None:
            self.realm = self.DEFAULT_REALM




"""Tests for distutils.command.upload."""
import sys
import os
import unittest

from collective.dist.mupload import mupload
from distutils.core import Distribution

from distutils.tests import support
from collective.dist.tests.test_pypirc import PYPIRC, PyPIRCCommandTestCase 

class uploadTestCase(PyPIRCCommandTestCase):

    def test_finalize_options(self):

        # new format
        f = open(self.rc, 'w')
        f.write(PYPIRC)
        f.close()

        dist = Distribution()
        cmd = mupload(dist)
        cmd.finalize_options()
        for attr, waited in (('username', 'me'), ('password', 'secret'),
                             ('realm', 'pypi'), 
                             ('repository', 'http://pypi.python.org/pypi')):
            self.assertEquals(getattr(cmd, attr), waited)

    
def test_suite():
    return unittest.makeSuite(uploadTestCase)

if __name__ == "__main__":
    unittest.main(defaultTest="test_suite")



# -*- coding: utf-8 -*-
# Copyright (C) 2008 Ingeniweb

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING. If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
This module contains the tool of iw.dist
"""
import os
from setuptools import setup, find_packages

version = '0.2.0'

README = os.path.join(os.path.dirname(__file__), 
          'collective', 'dist', 'docs', 'README.txt')
README = open(README).read() + '\n\n'
CHANGES = open('CHANGES').read() + '\n\n' 

long_description = README + CHANGES

setup(name='collective.dist',
      version=version,
      description="Distutils commands to upload files to several servers",
      long_description=long_description,
      # Get more strings from http://www.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
      keywords='pypi setuptools distutils',
      author='Tarek Ziade',
      author_email='support@ingeniweb.com',
      url='http://svn.plone.org/svn/collective/collective.dist',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['collective'],
      include_package_data=True,
      zip_safe=False,
      # uncomment this to be able to run tests with setup.py
      #test_suite = "collective.dist.tests.test_distdocs.test_suite",
      install_requires=[
          'setuptools',
          # -*- Extra requirements: -*-
      ],
      tests_require=['zope.testing'],
      entry_points = {
         "distutils.commands": [
            "mregister = collective.dist.mregister:mregister",
            "mupload = collective.dist.mupload:mupload"]}
      )

